
from Bio.SeqIO import parse
from snappy.src.seq_processing import gen_variants, letter_codes, letter_codes_rev, letter_codes_to_check
import re    
from scipy.stats import mode
import pandas as pd
from itertools import product
import polars as pl
from tqdm import tqdm
import numpy as np
from scipy.stats import chi2_contingency

from Bio.Seq import reverse_complement

import warnings
warnings.filterwarnings('ignore')

MODKIT_COLNAMES = (
        'contig',
        'start',
        'end',
        'mod_type',
        'score',
        'strand',
        'start_pos',
        'end_pos',
        'color',
        'n_valid',
        'frac_mod',
        'n_mod',
        'n_canon',
        'n_other',
        'n_delete',
        'n_fail',
        'n_diff',
        'n_nocall',
    )
    
canonical_bases = ['A', 'C', 'G', 'T']

re_to_degenerate = {
    '[A,]': 'A',
    '[C,]': 'C',
    '[G,]': 'G',
    '[T,]': 'T',
    '[A,C,]': 'M',
    '[A,G,]': 'R',
    '[A,T,]': 'W',
    '[C,G,]': 'S',
    '[C,T,]': 'Y',
    '[G,T,]': 'K',
    '[A,C,G,]': 'V',
    '[A,C,T,]': 'H',
    '[A,G,T,]': 'D',
    '[C,G,T,]': 'B',
    '[A,C,G,T,]': 'N',
}

degenerate_weght = {
    'A': 1,
    'C': 1,
    'G': 1,
    'T': 1,
    'M': 2,
    'R': 2,
    'W': 2,
    'S': 2,
    'Y': 2,
    'K': 2,
    'V': 3,
    'H': 3,
    'D': 3,
    'B': 3,
    'N': 4,
    '.': 4,
}



degenerate_to_re  = {
    re_to_degenerate[k]: k for k in re_to_degenerate
}

degenerate_bases = list(degenerate_to_re.keys())

OFFSETS_RANGE = list(range(0,12))

def add_contexts(df, reference):
    print('Adding contexts...')
    contexts = []
    for i in tqdm(range(len(df))):
        
        contig, start = str(df['contig'][i]), df['start'][i]
        if df['strand'][i] == '+':
            contexts.append(reference[contig][start - 15:start + 16])
        elif df['strand'][i]== '-':
            contexts.append(
                reverse_complement(reference[contig][start - 15:start + 16])
            )


    df = df.with_columns(
        pl.Series(name="context", values=contexts),
    )
    return df



def unite_motifs(m_list):    
    m_list = np.array([[s for s in m] for m in m_list]).T
    
    keys = [tuple(sorted(list(set(row)))) for row in m_list]
    
    new_motif = ''
    for k in keys:
        try:
            new_motif += letter_codes_rev[k]
        except KeyError:
            new_motif += k[0]
    return new_motif


def gen_kmers(repeat=3):
    
    kmers = [''.join(m) for m in product(canonical_bases, repeat=repeat)]
    return kmers 

kmers = gen_kmers()

def get_kmer_pair_keys(kmers):
    
    keys = list(product(kmers, repeat=2))
    return keys

def parse_modkit(file,):

    print('Read modkit table...')

    modkit_df = pl.read_csv(
        file, 
        separator = '\t', 
        has_header = False,
        new_columns = MODKIT_COLNAMES,
    )

    return modkit_df


def parse_fasta(file):
    fasta = parse(file, 'fasta')
    reference = {}
    for rec in fasta:
        reference[rec.description.split(' ')[0]] = str(rec.seq)
    return reference
    


def construct_control_graph(reference):

    kmers = gen_kmers()


    dict_freq_cntr = {}
    edge_count_cntr = 0

    print('Constructing control graph...')
    for offset in tqdm(OFFSETS_RANGE):
        
        dict_freq_cntr[offset] = {
            kmer_key: 0 for kmer_key in get_kmer_pair_keys(kmers)
        }
        
        
        for contig in reference.keys():
            for ref in (reference[contig], reverse_complement(reference[contig])):
                for i in range(len(ref)-4-offset):

                    source = ref[i:i+3]
                    target = ref[i+1+offset:i+4+offset]
                    try:
                        dict_freq_cntr[offset][(source, target)] += 1
                        edge_count_cntr += 1
                    except KeyError:
                        continue

    
    return dict_freq_cntr, edge_count_cntr


def get_dict_freq_exp(df, OFFSETS_RANGE):


    dict_freq_exp = {}
    edge_count_exp = 0

    print('Constructing exp graph...')
    for offset in tqdm(OFFSETS_RANGE):

        dict_freq_exp[offset] = {
            kmer_key: 0 for kmer_key in get_kmer_pair_keys(kmers)
        }

        for read in df['context']:

            for i in range(0 + offset, len(read) - 4 - offset):

                source = read[i:i+3]
                target = read[i+1+offset:i+4+offset]
                try:
                    dict_freq_exp[offset][(source, target)] += 1
                    edge_count_exp += 1
                except KeyError:
                    continue
                    
    return dict_freq_exp


def get_chi_stats(dict_freq_exp, dict_freq_cntr, min_freq_ratio = 2):
    

    chi_stat_tables = {}
    kmers = gen_kmers()
    
    print(f'Search for differential edges...')
    for _offset in tqdm(OFFSETS_RANGE):
        chi_stat_table = pd.DataFrame(index=kmers, columns=kmers)

        edge_count_exp = np.sum(list(dict_freq_exp[_offset].values()))
        edge_count_cntr = np.sum(list(dict_freq_cntr[_offset].values()))

        for k1 in chi_stat_table.columns:
            for k2 in chi_stat_table.index:
                
                f1, s1, f2, s2 = dict_freq_exp[_offset][(k1, k2)], edge_count_exp, dict_freq_cntr[_offset][(k1, k2)], edge_count_cntr

                

                if (f1/s1) / (f2/s2) < min_freq_ratio:
                    statistics = 0
                
                else:
                    _2x2_table = [
                        [f1, s1],
                        [f2, s2],
                    ]

                    try:
                        statistics = chi2_contingency(
                            _2x2_table
                        )[0]
                    except ValueError:
                        statistics = 0

                chi_stat_table[k1][k2] = statistics

        chi_stat_tables[_offset] = chi_stat_table
    return chi_stat_tables
        


def get_initial_max_edge(chi_stat_tables):


    max_edge = None
    max_confidence = -1

    for offset in OFFSETS_RANGE[:]:
        chi_stat_table = chi_stat_tables[offset]

        for k1 in chi_stat_table.columns:
            for k2 in chi_stat_table.index:
                confidence = chi_stat_table[k1][k2]
                if confidence > max_confidence:

                    max_edge = ((k1, k2), confidence, offset)
                    max_confidence = confidence

    return max_edge




def get_re_template(k1, k2,offset):
    
    if offset == 0:
        re_template = k1 + k2[-1:]
    elif offset == 1:
        re_template = k1 + k2[-2:]
    else:
        re_template = k1 + '.' * int((offset - len(k1)) + 1) + k2
        
    return re_template


def get_best_positions(pattern, contexts):
    
    p = re.compile(pattern)
    starts = []
    ends = []
    
    for context in contexts:
        
        for m in p.finditer(context):
            starts.append(m.start())
            ends.append(m.end())
    
    return mode(starts)[0], mode(ends)[0]



def get_modality(sdf, template, upper_thrs = 40, lower_thrs = 10, metric_thrs=0.87):
    
    t_variants = gen_variants(template)
    dists = []
    lens = []
    
    for v in t_variants:
        ssdf = sdf.filter(sdf['context'].str.contains(v))
        dists.append(list(ssdf['frac_mod']))
        lens.append(len(ssdf))
    
    max_repr = max(lens)
    dist = []
    for i, d in enumerate(dists):
        if lens[i] == 0:
            return False, 0.0
        dist += d * int(np.round(max_repr/lens[i]))

    dist = np.array(dist)
    
    if len(dist) == 0:
        return False, 0.0
    
    upper = len(dist[dist >= upper_thrs])
    lower = len(dist[dist <= lower_thrs])
    
    try:
        metric = upper/(upper + lower)
    except ZeroDivisionError:
        return False, 0.0
    
    if metric > metric_thrs:
        return True, metric
    
    else:
        return False, metric
    

def get_adjacent_positions(template):
    
    adjacent_positions = set([])
    for i, base in enumerate(template):
        if i == 0 or i == len(template) - 1:
            continue
        
        if base != '.': continue
        if template[i+1] != '.' or template[i-1] != '.':
            adjacent_positions.add(i)
            
    return sorted(list(adjacent_positions))
    
def get_non_N_positions(template):
    non_N_positions = []
    
    for i, base in enumerate(template):
        if base != '.':
            non_N_positions.append(i)
            
    return non_N_positions


def get_re_from_degenerate(template):
    
    new_template = ''
    for i, base in enumerate(template):
        if base == '.' or base in canonical_bases:
            new_template += base
        else:
            new_template += degenerate_to_re[base]
    return new_template        
    
def get_degenerate_from_re(re_template):
    
    new_template = re_template
    
    for key in re_to_degenerate:
        new_template = new_template.replace(key, re_to_degenerate[key])
    return new_template
    
    





def gen_position_specific_variants(template, pos):

    symbol = template[pos]

    s_templates = []
    for t in letter_codes_to_check[symbol]:
        s_templates.append(
            template[:pos] + t + template[pos+1:]
        )
    return s_templates


def is_degenerate(base):
    if base == '.' or base == 'N': return True
    return False

def curate_1_links(template, df,):

    positions_to_curate = []
    for i, base in enumerate(template):

        if  i == 0 or i == 31:
            continue

        if is_degenerate(base) and template[i-1] != '.' and template[i+1] !='.':
            positions_to_curate.append(i)

    if len(positions_to_curate) == 0:
        return template

    sdf = df.filter(df['context'].str.contains(template))
    
    for p in positions_to_curate:
        bases = tuple(sorted(list(set([seq[p] for seq in sdf['context']]))))
        new_base = letter_codes_rev[bases]


        template = template[:p] + new_base + template[p+1:]

    return template





def curate_motif(template, central_base, modkit_df, metric_thrs=0.87):
    

    if template[15] != central_base:
        template = template[:15] + central_base + template[16:]

    _modkit_local = modkit_df.clone()
    sdf = _modkit_local.filter(_modkit_local['context'].str.contains(template))
    is_uniformic, metric = get_modality(sdf, get_degenerate_from_re(template))
    print(metric)
    
    print('Initial template:', template)
    
    # extend template if required
    while is_uniformic == False:
        
        adjacent_positions = get_adjacent_positions(template)
        non_N_positions = get_non_N_positions(template)
        
        temp_sdf = _modkit_local.filter(_modkit_local['context'].str.contains(template))
        
        hits = []
        
        for pos in adjacent_positions:
            for s in degenerate_bases:
                
                curated_template = template[:pos] + s + template[pos+1:]
                re_curated_template = get_re_from_degenerate(curated_template)
                
                sdf = temp_sdf.filter(temp_sdf['context'].str.contains(re_curated_template))
                is_uniformic, metric = get_modality(sdf, curated_template)

                hits.append(
                    (curated_template, sum([degenerate_weght[base] for base in curated_template]), metric, is_uniformic)
                )

        if True in [p[3] for p in hits]:
            
            hits = [p for p in hits if p[3] == True]
            template = sorted(hits, reverse=True, key=lambda x: x[2])[0][0]
            is_uniformic = True
            
        else:
            template = sorted(hits, reverse=True, key=lambda x: x[2])[0][0]
        
    print('Uniformic template:', template)
    print()
    
    
    # iteratively check non-N positions
    print('Iterative correction...')
    non_N_positions = get_non_N_positions(template)
    for pos in non_N_positions:

        if pos == 15:
            continue
        pos_hits = []

        for s in degenerate_bases:
            if s in canonical_bases:
                continue

            curated_template = template[:pos] + s + template[pos+1:]
            re_curated_template = get_re_from_degenerate(curated_template)

            sdf = _modkit_local.filter(_modkit_local['context'].str.contains(re_curated_template))
            is_uniformic, metric = get_modality(sdf, curated_template)

            print(curated_template, re_curated_template, is_uniformic, np.round(metric, 3))

            if is_uniformic:
                pos_hits.append(
                    (curated_template, sum([degenerate_weght[base] for base in curated_template]))
                )

        if len(pos_hits) > 0:
            template = sorted(pos_hits, reverse=True, key=lambda x: x[1])[0][0]
        
        
        subvariant_hits = []
        for t in gen_position_specific_variants(template, pos):
            t_sdf = modkit_df.filter(modkit_df['context'].str.contains(get_re_from_degenerate(t)))
            is_uniformic, metric = get_modality(t_sdf, t)

            print (t, round(metric, 3))
            if metric >= metric_thrs:
                subvariant_hits.append(t)
        template = unite_motifs(subvariant_hits)


        corrected_segment = template[:pos + 1] + '.'*(31 - pos - 1)
        print(f'Local filtering for position {pos + 1}...')
        _modkit_local = _modkit_local.filter(_modkit_local['context'].str.contains(get_re_from_degenerate(corrected_segment)))

        

    
    print('Corrected motif:', template)
    
    return template




from snappy.src.methods import get_re_from_degenerate
import polars as pl
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns 

def prepare_output(results, modkit_df):
    mod_decode = {'a' : '6mA', 'm' : '5mC', '21839' : '4mC'}
    summary_table = {"Motif": [], 
                    "Modification type": [], 
                    "Num positions": [], 
                    "Median FracMod": []}
    mods = []
    regular_code = {'Site' : [], 'RegExp' : []}
    
    for mod in results.keys():

        modkit_df_sub = modkit_df.filter(modkit_df['mod_type'] == mod)
        
        for site,_,_ in results[mod]:
            
            modkit_df_motif = modkit_df_sub.filter(modkit_df_sub['context'].str.contains(get_re_from_degenerate(site)))
            site_rename = f"{site[:15]}{mod_decode[mod]}{site[16:]}".strip(".*").replace('.', 'N')
            
            regular_code['Site'].append(site_rename)
            regular_code['RegExp'].append(get_re_from_degenerate(site))
            
            summary_table["Motif"].append(site_rename)
            summary_table["Modification type"].append(mod_decode[mod])
            summary_table["Num positions"].append(len(modkit_df_motif))
            summary_table["Median FracMod"].append(np.round(np.median(list(modkit_df_motif['frac_mod'])), 2))
            modkit_df_motif = modkit_df_motif.with_columns(
                                        pl.Series(name="motif", values=[site_rename for i in range(len(modkit_df_motif))]),
                                        )
            modkit_df_motif = modkit_df_motif.with_columns(
                                        pl.Series(name="new_mod", values=[mod_decode[mod] for i in range(len(modkit_df_motif))]),
                                        )
            
            mods.append(modkit_df_motif)
    # Summary table
    summary_table = pl.from_dict(summary_table)
    mods = pl.concat(mods, how="align")

    new_names = {'contig' : 'Contig', 
                'start' : 'Coordinate', 
                'strand' : 'Strand', 
                'new_mod' : 'Modification', 
                'n_valid' : 'Coverage', 
                'motif' : 'Motif'}

    results_table =  mods[['contig', 'start', 'strand', 'new_mod', 'frac_mod', 'n_valid', 'motif']].rename(new_names)
    
    regular_code = pl.from_dicts(regular_code)

    return summary_table, results_table, regular_code

def create_viz(results, modkit_df, out):
    
    mod_decode = {'a' : '6mA', 'm' : '5mC', '21839' : '4mC'}

    for mod in results.keys():

        modkit_df_sub = modkit_df.filter(modkit_df['mod_type'] == mod)
        
        for site,_,_ in results[mod]:
        
            modkit_df_motif = modkit_df_sub.filter(modkit_df_sub['context'].str.contains(get_re_from_degenerate(site)))
            site_rename = f"{site[:15]}{mod_decode[mod]}{site[16:]}".strip(".*").replace('.', 'N')
            num_contig = len(set(modkit_df_motif['contig']))

            fig, axs = plt.subplots(num_contig, 1, figsize=(12, len(set(modkit_df_motif['contig']))*2))
            c = 0
            
            for contig in np.sort(list(set(modkit_df_motif['contig']))):
                if num_contig == 1: 
                    ax=axs
                else:
                    ax=axs[c]

                contig_subset = modkit_df_motif.filter(modkit_df_motif['contig'] == contig)
                sns.lineplot(x=contig_subset['start'], y=contig_subset['frac_mod'], linewidth=.5, c='grey', ax=ax)
                ax.set_ylabel('Fraction modified, %')
                ax.set_xlabel(f'Position in {contig}')
                ax.set_title(contig)
                ax.set_ylim(0, 100)
                ax.set_xlim(np.min(list(contig_subset['start']))-100, np.max(list(contig_subset['start']))+100)
                ax.spines['right'].set_visible(False)
                ax.spines['top'].set_visible(False)
                c += 1
            fig.suptitle(site_rename, fontsize=14)
            plt.tight_layout()
            plt.savefig(f'{out}/VIZ/{site_rename}_localization.pdf', bbox_inches='tight')

            fig, axs = plt.subplots(1, 1)
            sns.histplot(modkit_df_motif['frac_mod'], ax=axs, color='grey')
            axs.set_title(site_rename, fontsize=14)
            axs.set_xlabel('Fraction modified, %')
            axs.spines['right'].set_visible(False)
            axs.spines['top'].set_visible(False)
            sns.despine(trim=True)
            plt.savefig(f'{out}/VIZ/{site_rename}_FracMod.pdf', bbox_inches='tight')


def main():
    import argparse
    import sys
    import os
    from snappy.src.methods import parse_modkit, parse_fasta, add_contexts, construct_control_graph
    from snappy.src.main_stream import enrichment
    from snappy.src.generate_output import prepare_output, create_viz
    import numpy as np

    MIN_NUM_CONTEXTS = 30
    FRAC_MOD_THR = 80
    PERCENTILE = 0.1
    MIN_COV_THR = 10

    parser = argparse.ArgumentParser()

    parser.add_argument('-mk_bed', 
                        type=str, 
                        required=True, 
                        help='BED-file resulting from "modkit pileup"')
    parser.add_argument('-genome', 
                        type=str, 
                        required=True, 
                        help='Genome in FASTA format')
    parser.add_argument('-outdir',
                        type=str,
                        help='Output directory',
                        default='./Snappy_output')
    
    
    if len(sys.argv)==1:
        parser.print_help(sys.stderr)
        sys.exit(1)

    args = parser.parse_args()


    out = args.outdir
    reference = parse_fasta(args.genome)

    try:        
        os.mkdir(out)
    
    except FileExistsError:

        print('The output directory already exists!')
        sys.exit()

    print('                       RUNNING ...                        ')
    print('===================================================================')

    modkit_df = parse_modkit(args.mk_bed)
    modkit_df = modkit_df.filter(modkit_df['n_valid'] > modkit_df['n_diff'])

    COV_THR = max(
        [int(np.percentile(modkit_df['n_valid'], PERCENTILE)), MIN_COV_THR]
    )
    print(f'Coverage threshold: {COV_THR}')

    modkit_df = modkit_df.filter(modkit_df['n_valid'] > COV_THR)
    modkit_df = add_contexts(modkit_df, reference)
    
    modkit_df = modkit_df.filter(modkit_df['context'].str.contains('.'*31))


    dict_freq_cntr, edge_count_cntr = construct_control_graph(reference)


    a6 = modkit_df.filter(modkit_df['mod_type'] == 'a')
    m5 = modkit_df.filter(modkit_df['mod_type'] == 'm')
    m4 = modkit_df.filter(modkit_df['mod_type'] == '21839')

    results = {}

    df = a6.filter(a6['frac_mod'] > FRAC_MOD_THR)
    results['a'] = enrichment(df, a6, dict_freq_cntr, 'A', MIN_NUM_CONTEXTS=MIN_NUM_CONTEXTS)


    df = m5.filter(m5['frac_mod'] > FRAC_MOD_THR)
    results['m'] = enrichment(df, m5, dict_freq_cntr, 'C', MIN_NUM_CONTEXTS=MIN_NUM_CONTEXTS)


    df = m4.filter(m4['frac_mod'] > FRAC_MOD_THR)
    results['21839'] = enrichment(df, m4, dict_freq_cntr, 'C', MIN_NUM_CONTEXTS=MIN_NUM_CONTEXTS)


    if len(results['a']) + len(results['m']) + len(results['21839']) == 0:
        print('Methylation motifs were not identified!')

    else:
        summary_table, results_table, regular_code = prepare_output(results, modkit_df)


        print('                       PREPARING THE OUTPUT                        ')
        print('===================================================================')
        
        summary_table.write_csv(f'{out}/Summary_table.tsv', separator="\t")
        results_table.write_csv(f'{out}/Results_table.tsv', separator="\t")

        # Create advanced out
        try:        
            os.mkdir(f'{out}/advanced_res')
        
        except FileExistsError:

            print('The output directory already exists!')

        regular_code.write_csv(f'{out}/advanced_res/Reg_exp.tsv', separator="\t")
        modkit_df.write_csv(f'{out}/advanced_res/Modkit_tab.tsv', separator="\t")

        os.mkdir(out + '/VIZ/')
        create_viz(results, modkit_df, out)

        
        print('                               DONE!                               ')
        print('===================================================================')

if __name__ == "__main__":
    main()



from snappy.src.methods import get_dict_freq_exp, get_chi_stats, get_initial_max_edge, get_re_template, get_best_positions, curate_motif, get_re_from_degenerate
from scipy.stats import chi2_contingency

def get_chi_stat_ind(template, df, df_cntr):

    sdf = df.filter(df['context'].str.contains(get_re_from_degenerate(template)))
    sdf_cntr = df_cntr.filter(df_cntr['context'].str.contains(get_re_from_degenerate(template)))

    f1, s1, f2, s2 = len(sdf), len(df), len(sdf_cntr), len(df_cntr)

    return chi2_contingency(
        [[f1, s1], [f2, s2]]
    )



def enrichment(df, df_cntr, dict_freq_cntr, central_base, MIN_NUM_CONTEXTS=50, OFFSETS_RANGE=list(range(12)), MAX_MOTIFS=100):

    found_templates = []
    found_templates_cnt = []
    
    df_ = df.clone()

    iteration = 1
    print('\n')
    print(f'-----ITERATION {iteration}: {len(df)} unexplained contexts in the data-----')


    for i in range(MAX_MOTIFS):
        if len(df) < MIN_NUM_CONTEXTS:
            break
        dict_freq_exp = get_dict_freq_exp(df, OFFSETS_RANGE)
        chi_stat_tables = get_chi_stats(dict_freq_exp, dict_freq_cntr)
        initial_max_edge = get_initial_max_edge(chi_stat_tables)
        print('initial max edge:', initial_max_edge)

        re_template = get_re_template(*initial_max_edge[0], initial_max_edge[2])

        motif_positions = get_best_positions(re_template, df['context'])
        extended_re_template = '.'*(motif_positions[0]) + re_template + '.'*(31 - motif_positions[1])
        
        sdf = df.filter(df['context'].str.contains(re_template))
        if len(sdf) < MIN_NUM_CONTEXTS: 
            df = df.filter(df['context'].str.contains(get_re_from_degenerate(re_template)[1:-1]) == False)
            print('Skipped!')            
            continue

        sig = False
        for t in found_templates:
            if t.replace('.', '') in extended_re_template:
                print('Skipped as a submotif!')
                sig = True
                break
        
        if sig == False:
            curated_template = curate_motif(extended_re_template, central_base, df_cntr)

            
            sdf = df.filter(df['context'].str.contains(get_re_from_degenerate(curated_template)))
            sdf_cntr = df_cntr.filter(df_cntr['context'].str.contains(get_re_from_degenerate(curated_template)))

            f1, s1, f2, s2 = len(sdf), len(df), len(sdf_cntr), len(df_cntr)

            matrix = [[f1, s1], [f2, s2]]
            
            
            
            chi_res = chi2_contingency(
                matrix
            )

            print('Confidence:', round(chi_res[0], 1))
            
            if matrix[0][0] > MIN_NUM_CONTEXTS and chi_res[0] > 100:    
                found_templates.append(curated_template.replace('N', '.'))
                
                found_templates_cnt.append(
                    (
                        curated_template.replace('N', '.'), 
                        f1,
                        chi_res[0],
                    )
                )
                
            

        

        
        prev_len = len(df)
        df = df.filter(df['context'].str.contains(get_re_from_degenerate(curated_template)[1:-1]) == False)
        post_len = len(df)
        if prev_len == post_len:
            df = df.filter(df['context'].str.contains(get_re_from_degenerate(re_template)[1:-1]) == False)
        

        if len(df) < MIN_NUM_CONTEXTS:
            break
        print('\n')
        iteration += 1
        print(f'-----ITERATION {iteration}: {len(df)} unexplained contexts in the data-----')

    return found_templates_cnt


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ont-snappy",
    version="0.1.6",
    author="D.N. Konanov",
    author_email="konanovdmitriy@gmail.com",
    description="Nanopore-based methylation sites caller",
    long_description="snappy",
    long_description_content_type="",
    url="https://github.com/DNKonanov/ont-snappy",
    project_urls={
        "Bug Tracker": "https://github.com/DNKonanov/ont-snappy",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    include_package_data=True,
    packages=['snappy', 'snappy.src'],
    install_requires=[
        'biopython',
        'matplotlib',
        'scipy',
        'seaborn',
        'tqdm',
        'polars',
        'pandas'

    ],
    entry_points={
        'console_scripts': [
            'snappy=snappy.snappy:main'
        ]
    }
)
